﻿using Hotline.KnowledgeBase;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using Hotline.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class KnowledgeApplyController : BaseController
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeApplyRepository _knowledgeApplyRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IUserRepository _userRepository;
        private readonly IMapper _mapper;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeApplyRepository"></param>
        /// <param name="sessionContext"></param>
        /// <param name="userRepository"></param>
        /// <param name="mapper"></param>
        public KnowledgeApplyController(IKnowledgeRepository knowledgeRepository, IKnowledgeApplyRepository knowledgeApplyRepository, ISessionContext sessionContext, IUserRepository userRepository, IMapper mapper)
        {
            _knowledgeRepository = knowledgeRepository;
            _knowledgeApplyRepository = knowledgeApplyRepository;
            _sessionContext = sessionContext;
            _userRepository = userRepository;
            _mapper = mapper;
        }

        /// <summary>
        /// 知识申请-发起申请
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-add")]
        public async Task<string> ApplyAdd([FromBody] KnowledgeApplyDto dto)
        {
            var ka = _mapper.Map<KnowledgeApply>(dto);
            ka.CreationTime = DateTime.Now;
            ka.Status = EKnowledgeApplyStatus.Handling;
            return await _knowledgeApplyRepository.AddAsync(ka);
        }

        /// <summary>
        /// 知识申请-修改申请
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-update")]
        public async Task UpdateAdd([FromBody] UpdateKnowledgeApplyDto dto)
        {
            var ka = _mapper.Map<KnowledgeApply>(dto);
            await _knowledgeApplyRepository.UpdateAsync(ka);
        }

        /// <summary>
        ///知识申请-信息办理
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpPost("apply-handle")]
        public async Task ApplyHandle(string Id)
        {
            //查询申请信息
            var handle = await _knowledgeApplyRepository.GetAsync(Id);

            if (handle is null)
                throw UserFriendlyException.SameMessage("办理失败");
            handle.Status = EKnowledgeApplyStatus.Succeed;
            //修改办理信息
            await _knowledgeApplyRepository.UpdateAsync(handle);
        }

        /// <summary>
        ///知识申请- 撤销操作
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpPost("apply-handle/{Id}")]
        public async Task ApplyRevoke(string Id)
        {
            var handle = await _knowledgeApplyRepository.GetAsync(Id);
            if (handle is null)
                throw UserFriendlyException.SameMessage("申请信息撤销失败");
            handle.Status = EKnowledgeApplyStatus.Revert;
            await _knowledgeApplyRepository.UpdateAsync(handle);
        }

        /// <summary>
        /// 知识申请-退回操作
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpPost("apply-return")]
        public async Task ApplyReturn(string Id)
        {
            //查询申请信息
            var handle = await _knowledgeApplyRepository.GetAsync(Id);
            if (handle is null)
                throw UserFriendlyException.SameMessage("退回失败");
            handle.Status = EKnowledgeApplyStatus.Failed;
            await _knowledgeApplyRepository.UpdateAsync(handle);
        }

        /// <summary>
        /// 知识申请-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("apply-info")]
        public async Task<DataKnowledgeApplyDto> ApplyInfo(string Id)
        {
            var handle = await _knowledgeApplyRepository.GetAsync(Id);
            if (handle is null)
                throw UserFriendlyException.SameMessage("申请信息查询失败");
            DataKnowledgeApplyDto dataKnowledgeApplyDto = new();
            _mapper.Map(handle, dataKnowledgeApplyDto);

            var knowledge = await _knowledgeRepository.GetAsync(p => p.Applies.Contains(Id));
            if (knowledge != null)
            {
                dataKnowledgeApplyDto.KBID = knowledge.Id;
                dataKnowledgeApplyDto.KBTitle = knowledge.Title;
            }

            return dataKnowledgeApplyDto;
        }

        #region 我的知识申请
        /// <summary>
        /// 我的知识申请--全部
        /// </summary>
        /// <returns></returns>
        [HttpPost("apply-allapply")]
        public async Task<PagedDto<KnowledgeApply>> AllApply([FromQuery] KnowledgeApplyPagedDto dto)
        {
            var (total, items) = await _knowledgeApplyRepository
                  .Queryable()
                  .Includes(it => it.User)
                  .Includes(it => it.SystemOrganize)
                  .Where(d => d.CreatorId == _sessionContext.RequiredUserId)
                  .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
                  .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
                  .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
                  .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
                  .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
                  .OrderByDescending(p => p.CreationTime)
                  .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApply>(total, _mapper.Map<IReadOnlyList<KnowledgeApply>>(items));
        }

        /// <summary>
        /// 我的知识申请--处理中
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-processingapply")]
        public async Task<PagedDto<KnowledgeApply>> ProcessingApply([FromQuery] KnowledgeApplyPagedDto dto)
        {
            var (total, items) = await _knowledgeApplyRepository
               .Queryable()
               .Includes(it => it.User)
               .Includes(it => it.SystemOrganize)
               .Where(d => d.CreatorId == _sessionContext.RequiredUserId && d.Status == EKnowledgeApplyStatus.Handling)
               .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
               .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
               .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
               .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
               .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
               .OrderByDescending(p => p.CreationTime)
               .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApply>(total, _mapper.Map<IReadOnlyList<KnowledgeApply>>(items));
        }

        /// <summary>
        /// 我的知识申请--已完成
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-endapply")]
        public async Task<PagedDto<KnowledgeApply>> EndApply([FromQuery] KnowledgeApplyPagedDto dto)
        {
            var (total, items) = await _knowledgeApplyRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Where(d => d.CreatorId == _sessionContext.RequiredUserId && d.Status == EKnowledgeApplyStatus.Succeed)
              .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
              .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
              .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
              .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
              .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApply>(total, _mapper.Map<IReadOnlyList<KnowledgeApply>>(items));
        }

        /// <summary>
        /// 我的知识申请--退回
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-returnapply")]
        public async Task<PagedDto<KnowledgeApply>> ReturnApply([FromQuery] KnowledgeApplyPagedDto dto)
        {
            var (total, items) = await _knowledgeApplyRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Where(d => d.CreatorId == _sessionContext.RequiredUserId && d.Status == EKnowledgeApplyStatus.Failed)
              .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
              .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
              .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
              .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
              .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApply>(total, _mapper.Map<IReadOnlyList<KnowledgeApply>>(items));
        }

        /// <summary>
        /// 我的知识申请--已撤销
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-revokeapply")]
        public async Task<PagedDto<KnowledgeApply>> RevokeApply([FromQuery] KnowledgeApplyPagedDto dto)
        {
            var (total, items) = await _knowledgeApplyRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Where(d => d.CreatorId == _sessionContext.RequiredUserId && d.Status == EKnowledgeApplyStatus.Revert)
              .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
              .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
              .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
              .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
              .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApply>(total, _mapper.Map<IReadOnlyList<KnowledgeApply>>(items));
        }
        #endregion

        #region 知识申请处理查询
        /// <summary>
        /// 申请处理--全部
        /// </summary>
        /// <returns></returns>
        [HttpPost("apply-allhandle")]
        public async Task<PagedDto<KnowledgeApplyPageDto>> AllHandle([FromQuery] KnowledgeHandlePagedDto dto)
        {
            var orgUser = await _userRepository.GetAsync(p => p.Id == _sessionContext.RequiredUserId);
            if (orgUser is null)
                throw UserFriendlyException.SameMessage("登录信息获取失败");

            var (total, items) = await _knowledgeApplyRepository
                 .Queryable()
                 .Includes(it => it.User)
                 .Includes(it => it.SystemOrganize)
                 .Where(p => p.DepartmentCode == orgUser.OrgCode)
                 .WhereIF(!string.IsNullOrEmpty(dto.CreateUserName), d => d.User.Name.Contains(dto.CreateUserName!))
                 .WhereIF(!string.IsNullOrEmpty(dto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(dto.CreateBMName!))
                 .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
                 .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
                 .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
                 .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
                 .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
                 .OrderByDescending(p => p.CreationTime)
                 .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApplyPageDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApplyPageDto>>(items));
        }

        /// <summary>
        /// 申请处理--处理中
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-processinghandle")]
        public async Task<PagedDto<KnowledgeApplyPageDto>> ProcessingHandle([FromQuery] KnowledgeHandlePagedDto dto)
        {
            var orgUser = await _userRepository.GetAsync(p => p.Id == _sessionContext.RequiredUserId);
            if (orgUser is null)
                throw UserFriendlyException.SameMessage("登录信息获取失败");

            var (total, items) = await _knowledgeApplyRepository
                .Queryable(false)
                .Includes(it => it.User)
                .Includes(it => it.SystemOrganize)
                .Where(p => p.DepartmentCode == orgUser.OrgCode)
                .Where(p => p.Status == EKnowledgeApplyStatus.Handling)
                .WhereIF(!string.IsNullOrEmpty(dto.CreateUserName), d => d.User.Name.Contains(dto.CreateUserName!))
                .WhereIF(!string.IsNullOrEmpty(dto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(dto.CreateBMName!))
                .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
                .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
                .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
                .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
                .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
                .OrderByDescending(p => p.CreationTime)
                .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApplyPageDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApplyPageDto>>(items));
        }

        /// <summary>
        /// 申请处理--已完成
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-endhandle")]
        public async Task<PagedDto<KnowledgeApplyPageDto>> EndHandle([FromQuery] KnowledgeHandlePagedDto dto)
        {
            var orgUser = await _userRepository.GetAsync(p => p.Id == _sessionContext.RequiredUserId);
            if (orgUser is null)
                throw UserFriendlyException.SameMessage("登录信息获取失败");

            var (total, items) = await _knowledgeApplyRepository
               .Queryable(false)
               .Includes(it => it.User)
               .Includes(it => it.SystemOrganize)
               .Where(p => p.DepartmentCode == orgUser.OrgCode)
               .Where(p => p.Status == EKnowledgeApplyStatus.Succeed)
               .WhereIF(!string.IsNullOrEmpty(dto.CreateUserName), d => d.User.Name.Contains(dto.CreateUserName!))
               .WhereIF(!string.IsNullOrEmpty(dto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(dto.CreateBMName!))
               .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
               .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
               .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
               .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
               .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
               .OrderByDescending(p => p.CreationTime)
               .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApplyPageDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApplyPageDto>>(items));
        }

        /// <summary>
        /// 申请处理--退回
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("apply-returnhandle")]
        public async Task<PagedDto<KnowledgeApplyPageDto>> ReturnHandle([FromQuery] KnowledgeHandlePagedDto dto)
        {
            var orgUser = await _userRepository.GetAsync(p => p.Id == _sessionContext.RequiredUserId);
            if (orgUser is null)
                throw UserFriendlyException.SameMessage("登录信息获取失败");

            var (total, items) = await _knowledgeApplyRepository
               .Queryable(false)
               .Includes(it => it.User)
               .Includes(it => it.SystemOrganize)
               .Where(p => p.DepartmentCode == orgUser.OrgCode)
               .Where(p => p.Status == EKnowledgeApplyStatus.Failed)
               .WhereIF(!string.IsNullOrEmpty(dto.CreateUserName), d => d.User.Name.Contains(dto.CreateUserName!))
               .WhereIF(!string.IsNullOrEmpty(dto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(dto.CreateBMName!))
               .WhereIF(!string.IsNullOrEmpty(dto.ApplyType), d => d.KnowledgeApplyType.ToString() == dto.ApplyType)
               .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "0", d => d.ExpiredTime < DateTime.Now)
               .WhereIF(!string.IsNullOrEmpty(dto.IsOvertime) && dto.IsOvertime == "1", d => d.ExpiredTime >= DateTime.Now)
               .WhereIF(dto.StartTime != null, d => d.CreationTime >= dto.StartTime)
               .WhereIF(dto.EndTime != null, d => d.CreationTime <= dto.EndTime)
               .OrderByDescending(p => p.CreationTime)
               .ToPagedListAsync(dto.PageIndex, dto.PageSize);

            return new PagedDto<KnowledgeApplyPageDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApplyPageDto>>(items));

        }
        #endregion
    }
}
